#include <joyos.h>

extern uint32_t matchStartTime;

//returns number of milliseconds since match started;
int getMatchTime() {
	return (int)(get_time()-matchStartTime);
}

int ballsAbleToBeCaptured(int _territory) {
	//return number of balls a territory that could be gathered from a territory
	//taking into account the number of balls left in the territory and the
	//rate limit
	return game.territories[_territory].remaining;
}

//uses a weighting function to return what the next best action would be
//called after an action is completed/times out
//returns the point with the highest value of the following weight function:
//
//               (Points to gain in action)
// Weight =  ---------------------------------  -  (# territories away)*(time est to move 1 territory)*(how precious time is)  -  (is enemy is territory)
//           (time est)*(how precious time is)
//
// Weight = "Point Potential"  -  "Travel Cost"  -  "Collision Avoidance"
//
// ************* Additional Terms: ******************
// LEVER: large negative if territory not owned
// GEARBOX: large negative if already owned territory
//
int getBestMove() {

	copy_objects();
	update_position();

	int currentTerritory = my_current_territory();
	int enemyTerritory = enemy_current_territory();
	float timeWeight = getMatchTime()/1000; //timeWeight goes up 1 every second

	int bestActionPointIndex = -1;
	float highestWeight = (-1E+30); //max low, first value compared will always be higher
	float calculatedWeight = -1;
	
	//for all gearboxes -- capturing
	for ( int i = 0; i < 6; i++ ) { //for all 6 territories
		
		calculatedWeight = ( POINTS_GAINED_CAPTURE ) / ( TIME_ESTIMATE_CAPTURE*timeWeight ); //"point potential"
		calculatedWeight -= ( get_territory_difference(currentTerritory,i) ) * ( TIME_ESTIMATE_TERRITORY*timeWeight ); //"travel cost"
		if ( enemyTerritory == i ) {
			//never want to go to a territory that the enemy is in
			calculatedWeight = DO_NOT_DO_THIS_CONSTANT; //"collision avoidance"
			continue;
		}
		if ( game.territories[i].owner == 8 ) { //I own territory
			calculatedWeight = DO_NOT_DO_THIS_CONSTANT;
			continue;
		}
		
		if ( calculatedWeight > highestWeight ) {
			highestWeight = calculatedWeight;
			bestActionPointIndex = i*N_TERRITORY_POINTS+TERRITORY_GEARBOX;
		}
	}
	
	//for all levers
	for ( int i = 0; i < 6; i++ ) { //for all 6 territories
		
		calculatedWeight = ( POINTS_GAINED_LEVER*ballsAbleToBeCaptured(i) ) / ( TIME_ESTIMATE_LEVER*timeWeight ); //"point potential"
		calculatedWeight -= ( get_territory_difference(currentTerritory,i) ) * ( TIME_ESTIMATE_TERRITORY*timeWeight ); //"travel cost"
		if ( enemyTerritory == i ) {
			//never want to go to a territory that the enemy is in
			calculatedWeight = DO_NOT_DO_THIS_CONSTANT; //"collision avoidance"
			continue;
		}
		if ( game.territories[i].owner != 8 ) { //I do not own territory
			calculatedWeight = DO_NOT_DO_THIS_CONSTANT;
			continue;
		}
		
		if ( calculatedWeight > highestWeight ) {
			highestWeight = calculatedWeight;
			bestActionPointIndex = i*N_TERRITORY_POINTS+TERRITORY_LEVER;
		}
	}
	
	//for all drop points
	//POINTS CHANGE BASED ON WHAT SIDE START OF
	int territoriesBlue[3] = {1,2,3};
	int territoriesRed[3] = {4,5,0};	
	int i;
	
	for ( int a = 0; a < 3; a++ ) {
	
		i = ( team == BLUE ) ? territoriesBlue[a] : territoriesRed[a];
		
		calculatedWeight = ( POINTS_GAINED_DROP*nBallsCollected ) / ( TIME_ESTIMATE_DROP*timeWeight ); //"point potential"
		calculatedWeight -= ( get_territory_difference(currentTerritory,i) ) * ( TIME_ESTIMATE_TERRITORY*timeWeight ); //"travel cost"
		if ( enemyTerritory == i ) {
			//never want to go to a territory that the enemy is in
			calculatedWeight = DO_NOT_DO_THIS_CONSTANT; //"collision avoidance"
			continue;
		}

		if ( nBallsCollected == 0 ) {
			//never want to go to a territory that the enemy is in
			calculatedWeight = DO_NOT_DO_THIS_CONSTANT; //"collision avoidance"
			continue;
		}
		
		if ( calculatedWeight > highestWeight ) {
			highestWeight = calculatedWeight;
			bestActionPointIndex = i*N_TERRITORY_POINTS+TERRITORY_CITY;
		}
	}
	
	return bestActionPointIndex;

}